function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//@flow
import * as React from "react";

const getPreviousEnabled = (currentPage
/*: number*/
) => currentPage > 0;

const getNextEnabled = (currentPage
/*: number*/
, totalPages
/*: number*/
) => currentPage + 1 < totalPages;

const getTotalPages = (totalItems
/*: number*/
, pageSize
/*: number*/
) => Math.ceil(totalItems / pageSize);

const getStartIndex = (pageSize
/*: number*/
, currentPage
/*: number*/
) => pageSize * currentPage;

const getEndIndex = (pageSize
/*: number*/
, currentPage
/*: number*/
, totalItems
/*: number*/
) => {
  const lastPageEndIndex = pageSize * (currentPage + 1);

  if (lastPageEndIndex > totalItems) {
    return totalItems - 1;
  }

  return lastPageEndIndex - 1;
};

const getPaginationState = ({
  /*: {
      totalItems: number,
      pageSize: number,
      currentPage: number,
  }*/

  totalItems,
  pageSize,
  currentPage
}) => {
  const totalPages = getTotalPages(totalItems, pageSize);
  return {
    totalPages,
    startIndex: getStartIndex(pageSize, currentPage),
    endIndex: getEndIndex(pageSize, currentPage, totalItems),
    previousEnabled: getPreviousEnabled(currentPage),
    nextEnabled: getNextEnabled(currentPage, totalPages)
  };
};
/*:: type CurrentPageReducerActions = {| type: "SET", page: number |} | {| type: "NEXT" | "PREV" |};*/


function usePagination({
  /*: {
      totalItems?: number,
      initialPage?: number,
      initialPageSize?: number,
  }*/

  totalItems = 0,
  initialPage = 0,
  initialPageSize = 0
} = {}) {
  const [pageSize, setPageSize] = React.useState(initialPageSize);
  const [currentPage, dispatch] = React.useReducer((state = initialPage, action = {}) => {
    switch (action.type) {
      case "SET":
        return action.page;

      case "NEXT":
        if (!getNextEnabled(state, getTotalPages(totalItems, pageSize))) {
          return state;
        }

        return state + 1;

      case "PREV":
        if (!getPreviousEnabled(state)) {
          return state;
        }

        return state - 1;

      default:
        return state;
    }
  }, initialPage);
  const paginationState = React.useMemo(() => getPaginationState({
    totalItems,
    pageSize,
    currentPage
  }), [totalItems, pageSize, currentPage]);
  return _objectSpread({
    setPage: React.useCallback((page
    /*: number*/
    ) => {
      dispatch({
        type: "SET",
        page
      });
    }, [dispatch]),
    setNextPage: React.useCallback(() => {
      dispatch({
        type: "NEXT"
      });
    }, [dispatch]),
    setPreviousPage: React.useCallback(() => {
      dispatch({
        type: "PREV"
      });
    }, [dispatch]),
    setPageSize: React.useCallback((pageSize
    /*: number*/
    , nextPage
    /*:: ?: number*/
    = 0) => {
      setPageSize(pageSize);
      dispatch({
        type: "SET",
        page: nextPage
      });
    }, [setPageSize]),
    currentPage,
    pageSize
  }, paginationState);
}
/*:: type PaginationProps = {|
    children: ($Call<typeof usePagination>) => React.Node,
    totalItems?: number,
    initialPage?: number,
    initialPageSize: number,
|};*/


function Pagination({
  /*: PaginationProps*/

  children,
  totalItems = 0,
  initialPage = 0,
  initialPageSize
}) {
  return children(usePagination({
    totalItems,
    initialPage,
    initialPageSize
  }));
}

Pagination.displayName = "Pagination";
export { getPaginationState, usePagination };
export default Pagination;