import { useState, useReducer, useMemo, useCallback } from 'react';

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _objectSpread$1(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty$1(target, key, source[key]);
    });
  }

  return target;
}

function _defineProperty$1(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
} //@flow

const getPreviousEnabled = (currentPage
/*: number*/
) => currentPage > 0;

const getNextEnabled = (currentPage
/*: number*/
, totalPages
/*: number*/
) => currentPage + 1 < totalPages;

const getTotalPages = (totalItems
/*: number*/
, pageSize
/*: number*/
) => Math.ceil(totalItems / pageSize);

const getStartIndex = (pageSize
/*: number*/
, currentPage
/*: number*/
) => pageSize * currentPage;

const getEndIndex = (pageSize
/*: number*/
, currentPage
/*: number*/
, totalItems
/*: number*/
) => {
  const lastPageEndIndex = pageSize * (currentPage + 1);

  if (lastPageEndIndex > totalItems) {
    return totalItems - 1;
  }

  return lastPageEndIndex - 1;
};

const getPaginationState = (_ref) => {
  let totalItems = _ref.totalItems,
      pageSize = _ref.pageSize,
      currentPage = _ref.currentPage;
  const totalPages = getTotalPages(totalItems, pageSize);
  return {
    totalPages,
    startIndex: getStartIndex(pageSize, currentPage),
    endIndex: getEndIndex(pageSize, currentPage, totalItems),
    previousEnabled: getPreviousEnabled(currentPage),
    nextEnabled: getNextEnabled(currentPage, totalPages)
  };
};
/*:: type CurrentPageReducerActions = {| type: "SET", page: number |} | {| type: "NEXT" | "PREV" |};*/


function usePagination() {
  let _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref2$totalItems = _ref2.totalItems,
      totalItems = _ref2$totalItems === void 0 ? 0 : _ref2$totalItems,
      _ref2$initialPage = _ref2.initialPage,
      initialPage = _ref2$initialPage === void 0 ? 0 : _ref2$initialPage,
      _ref2$initialPageSize = _ref2.initialPageSize,
      initialPageSize = _ref2$initialPageSize === void 0 ? 0 : _ref2$initialPageSize;

  const _React$useState = useState(initialPageSize),
        _React$useState2 = _slicedToArray(_React$useState, 2),
        pageSize = _React$useState2[0],
        setPageSize = _React$useState2[1];

  const _React$useReducer = useReducer(function () {
    let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialPage;
    let action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    switch (action.type) {
      case "SET":
        return action.page;

      case "NEXT":
        if (!getNextEnabled(state, getTotalPages(totalItems, pageSize))) {
          return state;
        }

        return state + 1;

      case "PREV":
        if (!getPreviousEnabled(state)) {
          return state;
        }

        return state - 1;

      default:
        return state;
    }
  }, initialPage),
        _React$useReducer2 = _slicedToArray(_React$useReducer, 2),
        currentPage = _React$useReducer2[0],
        dispatch = _React$useReducer2[1];

  const paginationState = useMemo(() => getPaginationState({
    totalItems,
    pageSize,
    currentPage
  }), [totalItems, pageSize, currentPage]);
  return _objectSpread$1({
    setPage: useCallback((page
    /*: number*/
    ) => {
      dispatch({
        type: "SET",
        page
      });
    }, [dispatch]),
    setNextPage: useCallback(() => {
      dispatch({
        type: "NEXT"
      });
    }, [dispatch]),
    setPreviousPage: useCallback(() => {
      dispatch({
        type: "PREV"
      });
    }, [dispatch]),
    setPageSize: useCallback(function (pageSize
    /*: number*/
    ) {
      let nextPage
      /*:: ?: number*/
      = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      setPageSize(pageSize);
      dispatch({
        type: "SET",
        page: nextPage
      });
    }, [setPageSize]),
    currentPage,
    pageSize
  }, paginationState);
}
/*:: type PaginationProps = {|
    children: ($Call<typeof usePagination>) => React.Node,
    totalItems?: number,
    initialPage?: number,
    initialPageSize: number,
|};*/


function Pagination(_ref3) {
  let children = _ref3.children,
      _ref3$totalItems = _ref3.totalItems,
      totalItems = _ref3$totalItems === void 0 ? 0 : _ref3$totalItems,
      _ref3$initialPage = _ref3.initialPage,
      initialPage = _ref3$initialPage === void 0 ? 0 : _ref3$initialPage,
      initialPageSize = _ref3.initialPageSize;
  return children(usePagination({
    totalItems,
    initialPage,
    initialPageSize
  }));
}

Pagination.displayName = "Pagination";

export default Pagination;
export { getPaginationState, usePagination };
